from .core.interpreter import Interpreter
from .castle.castle import Castle
from .castle.defense import Defense
from .map.bookmarks import Bookmarks
from .map.map import Map
from .map.movements import Movements
from .map.attack import Attack
from .map.support import Support
from .map.spy import Spy
from .misc.misc import Misc
from .army.lords import Lords
from .building.buildings import Buildings
from .building.repair import Repair
from .building.buildings_inventory import BuildingsInventory
from .building.wall import Wall
from .building.extension import Extension
from .building.build_items import BuildItems
from .equip.equip import Equip
from .production.soldiers import Soldiers
from .production.tools import Tools
from .production.hospital import Hospital
from .production.refinery import Refinery
from .alliance.alliance import Alliance
from .social.social import Social
from .shops.shop import Shop
from .shops.kings_market import KingsMarket
from .shops.bestseller import BestSeller
from .events.events import Events
from .events.beyond_the_horizon import BeyondTheHorizon
from .events.outer_realms import OuterRealms
from .events.imperial_patronage import ImperialPatronage






 

class GGXS(
    Interpreter,
    Castle,
    Bookmarks,
    Map,
    Attack,
    Movements,
    Misc,
    Lords,
    Defense,
    Soldiers,
    Tools,
    Buildings,
    Repair,
    BuildingsInventory,
    Wall,
    Extension,
    Alliance,
    Equip,
    Social,
    Support,
    Shop,
    KingsMarket,
    BestSeller,
    BuildItems,
    Events,
    BeyondTheHorizon,
    OuterRealms,
    ImperialPatronage,
    Spy,
    Hospital,
    Refinery
):
    pass